@@ -2,7 +2,6 @@
 {-# LANGUAGE GADTs #-}
 {-# LANGUAGE Rank2Types #-}
 {-# LANGUAGE TypeFamilies #-}
-{-# LANGUAGE TemplateHaskell #-}
 {-# LANGUAGE FlexibleContexts #-}
 {-# LANGUAGE DefaultSignatures #-}
 {-# LANGUAGE FlexibleInstances #-}
@@ -42,11 +41,10 @@ module Control.Lens.At
   ) where
 
 import Control.Applicative
+import Control.Lens.Each
+import Control.Lens.Traversal
 import Control.Lens.Lens
 import Control.Lens.Setter
-import Control.Lens.Type
-import Control.Lens.Internal.TupleIxedTH (makeAllTupleIxed)
-import Data.Aeson as Aeson
 import Data.Array.IArray as Array
 import Data.Array.Unboxed
 import Data.ByteString as StrictB
@@ -64,7 +62,6 @@ import Data.Set as Set
 import Data.Sequence as Seq
 import Data.Text as StrictT
 import Data.Text.Lazy as LazyT
-import Data.Traversable
 import Data.Tree
 import Data.Vector as Vector hiding (indexed)
 import Data.Vector.Primitive as Prim
@@ -105,7 +102,6 @@ type instance Index StrictT.Text = Int
 type instance Index LazyT.Text = Int64
 type instance Index StrictB.ByteString = Int
 type instance Index LazyB.ByteString = Int64
-type instance Index Aeson.Value = StrictT.Text
 
 -- $setup
 -- >>> :set -XNoOverloadedStrings
@@ -148,14 +144,14 @@ instance (Eq a, Hashable a) => Contains (HashSet a) where
 -- | This provides a common notion of a value at an index that is shared by both 'Ixed' and 'At'.
 type family IxValue (m :: *) :: *
 
--- | This simple 'AffineTraversal' lets you 'traverse' the value at a given
+-- | This simple 'Traversal' lets you 'traverse' the value at a given
 -- key in a 'Map' or element at an ordinal position in a list or 'Seq'.
 class Ixed m where
-  -- | This simple 'AffineTraversal' lets you 'traverse' the value at a given
+  -- | This simple 'Traversal' lets you 'traverse' the value at a given
   -- key in a 'Map' or element at an ordinal position in a list or 'Seq'.
   --
-  -- /NB:/ Setting the value of this 'AffineTraversal' will only set the value in the
-  -- 'Lens' if it is already present.
+  -- /NB:/ Setting the value of this 'Traversal' will only set the value in
+  -- 'at' if it is already present.
   --
   -- If you want to be able to insert /missing/ values, you want 'at'.
   --
@@ -362,13 +358,6 @@ instance Ixed LazyB.ByteString where
   {-# INLINE ix #-}
 
 
-type instance IxValue Aeson.Value = Aeson.Value
-instance Ixed Aeson.Value where
-  ix i f (Object o) = Object <$> ix i f o
-  ix _ _ v          = pure v
-  {-# INLINE ix #-}
-
-
 
 -- | 'At' provides a 'Lens' that can be used to read,
 -- write or delete the value associated with a key in a 'Map'-like
@@ -441,4 +430,43 @@ instance (Eq k, Hashable k) => At (HashSet k) where
     where mv = if HashSet.member k m then Just () else Nothing
   {-# INLINE at #-}
 
-makeAllTupleIxed
+
+-- | @'ix' :: 'Int' -> 'Traversal'' (a,a) a@
+type instance IxValue (a,a2) = a
+instance (a~a2) => Ixed (a,a2) where
+  ix = elementOf each
+
+-- | @'ix' :: 'Int' -> 'Traversal'' (a,a,a) a@
+type instance IxValue (a,a2,a3) = a
+instance (a~a2, a~a3) => Ixed (a,a2,a3) where
+  ix = elementOf each
+
+-- | @'ix' :: 'Int' -> 'Traversal'' (a,a,a,a) a@
+type instance IxValue (a,a2,a3,a4) = a
+instance (a~a2, a~a3, a~a4) => Ixed (a,a2,a3,a4) where
+  ix = elementOf each
+
+-- | @'ix' :: 'Int' -> 'Traversal'' (a,a,a,a,a) a@
+type instance IxValue (a,a2,a3,a4,a5) = a
+instance (a~a2, a~a3, a~a4, a~a5) => Ixed (a,a2,a3,a4,a5) where
+  ix = elementOf each
+
+-- | @'ix' :: 'Int' -> 'Traversal'' (a,a,a,a,a,a) a@
+type instance IxValue (a,a2,a3,a4,a5,a6) = a
+instance (a~a2, a~a3, a~a4, a~a5, a~a6) => Ixed (a,a2,a3,a4,a5,a6) where
+  ix = elementOf each
+
+-- | @'ix' :: 'Int' -> 'Traversal'' (a,a,a,a,a,a,a) a@
+type instance IxValue (a,a2,a3,a4,a5,a6,a7) = a
+instance (a~a2, a~a3, a~a4, a~a5, a~a6, a~a7) => Ixed (a,a2,a3,a4,a5,a6,a7) where
+  ix = elementOf each
+
+-- | @'ix' :: 'Int' -> 'Traversal'' (a,a,a,a,a,a,a,a) a@
+type instance IxValue (a,a2,a3,a4,a5,a6,a7,a8) = a
+instance (a~a2, a~a3, a~a4, a~a5, a~a6, a~a7, a~a8) => Ixed (a,a2,a3,a4,a5,a6,a7,a8) where
+  ix = elementOf each
+
+-- | @'ix' :: 'Int' -> 'Traversal'' (a,a,a,a,a,a,a,a,a) a@
+type instance IxValue (a,a2,a3,a4,a5,a6,a7,a8,a9) = a
+instance (a~a2, a~a3, a~a4, a~a5, a~a6, a~a7, a~a8, a~a9) => Ixed (a,a2,a3,a4,a5,a6,a7,a8,a9) where
+  ix = elementOf each
