@@ -33,6 +33,7 @@ module Control.Lens.Indexed
   , Conjoined(..)
   , Indexed(..)
   , (<.), (<.>), (.>)
+  , selfIndex
   , reindexed
   , icompose
   , indexing
@@ -77,6 +78,7 @@ import Control.Applicative
 import Control.Applicative.Backwards
 import Control.Monad (void, liftM)
 import Control.Monad.Trans.State.Lazy as Lazy
+import Control.Monad.Free
 import Control.Lens.Fold
 import Control.Lens.Getter
 import Control.Lens.Internal.Fold
@@ -110,6 +112,10 @@ infixr 9 <.>, <., .>
 -- | Compose an 'Indexed' function with a non-indexed function.
 --
 -- Mnemonically, the @<@ points to the indexing we want to preserve.
+--
+-- >>> let nestedMap = (fmap Map.fromList . Map.fromList) [(1, [(10, "one,ten"), (20, "one,twenty")]), (2, [(30, "two,thirty"), (40,"two,forty")])]
+-- >>> nestedMap^..(itraversed<.itraversed).withIndex
+-- [(1,"one,ten"),(1,"one,twenty"),(2,"two,thirty"),(2,"two,forty")]
 (<.) :: Indexable i p => (Indexed i s t -> r) -> ((a -> b) -> s -> t) -> p a b -> r
 (<.) f g h = f . Indexed $ g . indexed h
 {-# INLINE (<.) #-}
@@ -122,10 +128,26 @@ infixr 9 <.>, <., .>
 --
 -- @f '.' g@ (and @f '.>' g@) gives you the index of @g@ unless @g@ is index-preserving, like a
 -- 'Prism', 'Iso' or 'Equality', in which case it'll pass through the index of @f@.
+--
+-- >>> let nestedMap = (fmap Map.fromList . Map.fromList) [(1, [(10, "one,ten"), (20, "one,twenty")]), (2, [(30, "two,thirty"), (40,"two,forty")])]
+-- >>> nestedMap^..(itraversed.>itraversed).withIndex
+-- [(10,"one,ten"),(20,"one,twenty"),(30,"two,thirty"),(40,"two,forty")]
 (.>) :: (st -> r) -> (kab -> st) -> kab -> r
 (.>) = (.)
 {-# INLINE (.>) #-}
 
+-- | Use a value itself as its own index. This is essentially an indexed version of 'id'.
+--
+-- Note: When used to modify the value, this can break the index requirements assumed by 'indices' and similar,
+-- so this is only properly an 'IndexedGetter', but it can be used as more.
+--
+-- @
+-- 'selfIndex' :: 'IndexedGetter' a a b
+-- @
+selfIndex :: Indexable a p => p a fb -> a -> fb
+selfIndex f a = indexed f a a
+{-# INLINE selfIndex #-}
+
 -- | Remap the index.
 reindexed :: Indexable j p => (i -> j) -> (Indexed i a b -> r) -> p a b -> r
 reindexed ij f g = f . Indexed $ indexed g . ij
@@ -134,6 +156,10 @@ reindexed ij f g = f . Indexed $ indexed g . ij
 -- | Composition of 'Indexed' functions.
 --
 -- Mnemonically, the @\<@ and @\>@ points to the fact that we want to preserve the indices.
+--
+-- >>> let nestedMap = (fmap Map.fromList . Map.fromList) [(1, [(10, "one,ten"), (20, "one,twenty")]), (2, [(30, "two,thirty"), (40,"two,forty")])]
+-- >>> nestedMap^..(itraversed<.>itraversed).withIndex
+-- [((1,10),"one,ten"),((1,20),"one,twenty"),((2,30),"two,thirty"),((2,40),"two,forty")]
 (<.>) :: Indexable (i, j) p => (Indexed i s t -> r) -> (Indexed j a b -> s -> t) -> p a b -> r
 f <.> g = icompose (,) f g
 {-# INLINE (<.>) #-}
@@ -236,6 +262,11 @@ class Foldable f => FoldableWithIndex i f | f -> i where
 #endif
 
   -- | The 'IndexedFold' of a 'FoldableWithIndex' container.
+  --
+  -- 'ifolded'.'asIndex' is a fold over the keys of a 'FoldableWithIndex'.
+  --
+  -- >>> Data.Map.fromList [(2, "hello"), (1, "world")]^..ifolded.asIndex
+  -- [1,2]
   ifolded :: IndexedFold i (f a) a
   ifolded = conjoined folded $ \f -> coerce . getFolding . ifoldMap (\i -> Folding #. indexed f i)
   {-# INLINE ifolded #-}
@@ -436,7 +467,7 @@ ifoldlM f z0 xs = ifoldr f' return xs z0
 -- When you don't need access to the indices in the result, then 'Data.Foldable.toList' is more flexible in what it accepts.
 --
 -- @
--- 'Data.Foldable.toList' ≡ 'Data.List.map' 'fst' '.' 'itoList'
+-- 'Data.Foldable.toList' ≡ 'Data.List.map' 'snd' '.' 'itoList'
 -- @
 itoList :: FoldableWithIndex i f => f a -> [(i,a)]
 itoList = ifoldr (\i c -> ((i,c):)) []
@@ -456,6 +487,10 @@ itoList = ifoldr (\i c -> ((i,c):)) []
 -- @
 class (FunctorWithIndex i t, FoldableWithIndex i t, Traversable t) => TraversableWithIndex i t | t -> i where
   -- | Traverse an indexed container.
+  --
+  -- @
+  -- 'itraverse' ≡ 'itraverseOf' 'itraversed'
+  -- @
   itraverse :: Applicative f => (i -> a -> f b) -> t a -> f (t b)
 #ifndef HLINT
   default itraverse :: Applicative f => (Int -> a -> f b) -> t a -> f (t b)
@@ -695,6 +730,21 @@ instance TraversableWithIndex i (Magma i t b) where
   itraverse f (Magma i a)      = Magma i <$> f i a
   {-# INLINE itraverse #-}
 
+instance FunctorWithIndex i f => FunctorWithIndex [i] (Free f) where
+  imap f (Pure a) = Pure $ f [] a
+  imap f (Free s) = Free $ imap (\i -> imap (f . (:) i)) s
+  {-# INLINE imap #-}
+
+instance FoldableWithIndex i f => FoldableWithIndex [i] (Free f) where
+  ifoldMap f (Pure a) = f [] a
+  ifoldMap f (Free s) = ifoldMap (\i -> ifoldMap (f . (:) i)) s
+  {-# INLINE ifoldMap #-}
+
+instance TraversableWithIndex i f => TraversableWithIndex [i] (Free f) where
+  itraverse f (Pure a) = Pure <$> f [] a
+  itraverse f (Free s) = Free <$> itraverse (\i -> itraverse (f . (:) i)) s
+  {-# INLINE itraverse #-}
+
 -------------------------------------------------------------------------------
 -- Misc.
 -------------------------------------------------------------------------------
