@@ -6,6 +6,10 @@
 #ifdef TRUSTWORTHY
 {-# LANGUAGE Trustworthy #-}
 #endif
+#ifndef MIN_VERSION_base
+#define MIN_VERSION_base(x,y,z) 1
+#endif
+
 -----------------------------------------------------------------------------
 -- |
 -- Module      :  Control.Lens.Internal.Deque
@@ -33,7 +37,12 @@ import Control.Lens.Iso
 import Control.Lens.Lens
 import Control.Lens.Prism
 import Control.Monad
+#if MIN_VERSION_base(4,8,0)
+import Data.Foldable hiding (null)
+import qualified Data.Foldable as Foldable
+#else
 import Data.Foldable as Foldable
+#endif
 import Data.Function
 import Data.Functor.Bind
 import Data.Functor.Plus
