@@ -72,7 +72,7 @@ module Control.Lens.Plated
   , contexts, contextsOf, contextsOn, contextsOnOf
   , holes, holesOn, holesOnOf
   , para, paraOf
-  , (...)
+  , (...), deep
 
   -- * Compos
   -- $compos
@@ -99,7 +99,6 @@ import Control.Monad.Trans.Free as Trans
 import Control.MonadPlus.Free as MonadPlus
 #endif
 import qualified Language.Haskell.TH as TH
-import Data.Aeson
 import Data.Bitraversable
 import Data.Data
 import Data.Data.Lens
@@ -242,12 +241,6 @@ instance Traversable f => Plated (Cofree f a) where
 instance Plated (Tree a) where
   plate f (Node a as) = Node a <$> traverse f as
 
-instance Plated Value where
-  plate f (Object o) = Object <$> traverse f o
-  plate f (Array a) = Array <$> traverse f a
-  plate _ xs = pure xs
-  {-# INLINE plate #-}
-
 {- Default uniplate instances -}
 instance Plated TH.Exp
 instance Plated TH.Dec
@@ -266,6 +259,19 @@ infixr 9 ...
 l ... m = l . plate . m
 {-# INLINE (...) #-}
 
+
+-- | Try to apply a traversal to all transitive descendants of a 'Plated' container, but
+-- do not recurse through matching descendants.
+--
+-- @
+-- 'deep' :: 'Plated' s => 'Fold' s a                 -> 'Fold' s a
+-- 'deep' :: 'Plated' s => 'IndexedFold' s a          -> 'IndexedFold' s a
+-- 'deep' :: 'Plated' s => 'Traversal' s s a b        -> 'Traversal' s s a b
+-- 'deep' :: 'Plated' s => 'IndexedTraversal' s s a b -> 'IndexedTraversal' s s a b
+-- @
+deep :: (Conjoined p, Applicative f, Plated s) => Traversing p f s s a b -> Over p f s s a b
+deep = deepOf plate
+
 -------------------------------------------------------------------------------
 -- Children
 -------------------------------------------------------------------------------
