@@ -30,9 +30,10 @@ module Control.Monad.Error.Lens
 import Control.Applicative
 import Control.Lens
 import Control.Lens.Internal.Exception
-import Control.Monad.Error
+import Control.Monad
+import Control.Monad.Error.Class
 import Data.Functor.Plus
-import Data.Monoid
+import Data.Monoid (Monoid(..), First(..))
 import Data.Semigroup (Semigroup(..))
 
 ------------------------------------------------------------------------------
@@ -207,7 +208,7 @@ instance Handleable e m (Handler e m) where
 -- 'throwing' :: 'MonadError' e m => 'Prism'' e t -> t -> a
 -- 'throwing' :: 'MonadError' e m => 'Iso'' e t   -> t -> a
 -- @
-throwing :: MonadError e m => AReview e e t t -> t -> m x
+throwing :: MonadError e m => AReview e t -> t -> m x
 throwing l = reviews l throwError
 {-# INLINE throwing #-}
 
