@@ -1,19 +1,23 @@
 {-# LANGUAGE CPP #-}
 {-# LANGUAGE MagicHash #-}
 {-# LANGUAGE Rank2Types #-}
+#ifndef HLINT
 {-# LANGUAGE UnboxedTuples #-}
+#endif
 {-# LANGUAGE PatternGuards #-}
 {-# LANGUAGE FlexibleContexts #-}
 {-# LANGUAGE DeriveDataTypeable #-}
 {-# LANGUAGE ScopedTypeVariables #-}
 {-# LANGUAGE ExistentialQuantification #-}
+{-# LANGUAGE GADTs #-}
 #ifdef TRUSTWORTHY
 {-# LANGUAGE Trustworthy #-}
 #endif
+{-# OPTIONS_GHC -fno-full-laziness #-}
 -----------------------------------------------------------------------------
 -- |
 -- Module      :  Data.Data.Lens
--- Copyright   :  (C) 2012-2013 Edward Kmett, (C) 2006-2012 Neil Mitchell
+-- Copyright   :  (C) 2012-2014 Edward Kmett, (C) 2006-2012 Neil Mitchell
 -- License     :  BSD-style (see the file LICENSE)
 -- Maintainer  :  Edward Kmett <ekmett@gmail.com>
 -- Stability   :  experimental
@@ -43,18 +47,15 @@ module Data.Data.Lens
 
 import           Control.Applicative
 import           Control.Exception as E
-import           Control.Lens.Combinators
 import           Control.Lens.Internal.Context
 import           Control.Lens.Internal.Indexed
+import           Control.Lens.Lens
 import           Control.Lens.Setter
 import           Control.Lens.Traversal
 import           Control.Lens.Type
 import           Data.Data
 import           GHC.IO
-import           Unsafe.Coerce as Unsafe
 import           Data.Maybe
-
-#ifndef SAFE
 import           Data.Foldable
 import qualified Data.HashMap.Strict as M
 import           Data.HashMap.Strict (HashMap, (!))
@@ -63,11 +64,13 @@ import           Data.HashSet (HashSet)
 import           Data.IORef
 import           Data.Monoid
 import           GHC.Exts (realWorld#)
-#endif
 
+#ifdef HLINT
 {-# ANN module "HLint: ignore Eta reduce" #-}
 {-# ANN module "HLint: ignore Use foldl" #-}
 {-# ANN module "HLint: ignore Reduce duplication" #-}
+{-# ANN module "HLint: ignore Unused LANGUAGE pragma" #-}
+#endif
 
 -- $setup
 -- >>> :set -XNoOverloadedStrings
@@ -97,10 +100,10 @@ tinplate :: (Data s, Typeable a) => Traversal' s a
 tinplate f = gfoldl (step f) pure
 {-# INLINE tinplate #-}
 
-step :: (Applicative f, Typeable a, Data s) => (a -> f a) -> f (s -> r) -> s -> f r
-step f w s = w <*> case cast s of
-  Just a  -> unsafeCoerce <$> f a
-  Nothing -> tinplate f s
+step :: forall s a f r. (Applicative f, Typeable a, Data s) => (a -> f a) -> f (s -> r) -> s -> f r
+step f w s = w <*> case mightBe :: Maybe (Is s a) of
+  Just Data.Data.Lens.Refl -> f s
+  Nothing   -> tinplate f s
 {-# INLINE step #-}
 
 -------------------------------------------------------------------------------
@@ -113,12 +116,8 @@ step f w s = w <*> case cast s of
 --
 -- This is 'uniplate' with a more liberal signature.
 template :: forall s a. (Data s, Typeable a) => Traversal' s a
-#ifdef SAFE
-template = tinplate
-#else
 template = uniplateData (fromOracle answer) where
   answer = hitTest (undefined :: s) (undefined :: a)
-#endif
 {-# INLINE template #-}
 
 -- | Find descendants of type @a@ non-transitively, while avoiding computation of areas that cannot contain values of
@@ -131,14 +130,8 @@ uniplate = template
 
 -- | 'biplate' performs like 'template', except when @s ~ a@, it returns itself and nothing else.
 biplate :: forall s a. (Data s, Typeable a) => Traversal' s a
-#ifdef SAFE
-biplate f s
-  | typeOf (undefined :: s) == typeOf (undefined :: a) = pure s
-  | otherwise                                          = template f s
-#else
 biplate = biplateData (fromOracle answer) where
   answer = hitTest (undefined :: s) (undefined :: a)
-#endif
 {-# INLINE biplate #-}
 
 ------------------------------------------------------------------------------
@@ -153,7 +146,7 @@ instance Show (FieldException a) where
 
 instance Typeable a => Exception (FieldException a)
 
-lookupon :: Typeable a => LensLike' (Indexing Mutator) s a -> (s -> a) -> s -> Maybe (Int, Context a a s)
+lookupon :: Typeable a => LensLike' (Indexing Identity) s a -> (s -> a) -> s -> Maybe (Int, Context a a s)
 lookupon l field s = case unsafePerformIO $ E.try $ evaluate $ field $ s & indexing l %@~ \i (a::a) -> E.throw (FieldException i a) of
   Right _ -> Nothing
   Left e -> case fromException e of
@@ -268,7 +261,16 @@ onceUpon' field f s = k <$> indexed f i (field s) where
   ~(i, Context k _) = fromMaybe (error "upon': no index, not a member") (lookupon template field s)
 {-# INLINE onceUpon' #-}
 
-#ifndef SAFE
+-------------------------------------------------------------------------------
+-- Type equality
+-------------------------------------------------------------------------------
+
+data Is a b where
+  Refl :: Is a a
+
+mightBe :: (Typeable a, Typeable b) => Maybe (Is a b)
+mightBe = gcast Data.Data.Lens.Refl
+{-# INLINE mightBe #-}
 
 -------------------------------------------------------------------------------
 -- Data Box
@@ -320,7 +322,7 @@ insertHitMap box hit = fixEq trans (populate box) `mappend` hit where
   trans :: HitMap -> HitMap
   trans m = M.map f m where
     f x = x `mappend` foldMap g x
-    g x = M.lookupDefault (hit ! x) x m
+    g x = fromMaybe (hit ! x) (M.lookup x m)
 
 fixEq :: Eq a => (a -> a) -> a -> a
 fixEq f = go where
@@ -373,63 +375,49 @@ readCacheHitMap b@(DataBox kb _) = inlinePerformIO $
 -- Answers
 -------------------------------------------------------------------------------
 
-data Answer a
-  = Hit a
+data Answer b a
+  = b ~ a => Hit a
   | Follow
   | Miss
-  deriving (Eq,Ord,Show,Read)
-
-instance Functor Answer where
-  fmap f (Hit a) = Hit (f a)
-  fmap _ Follow  = Follow
-  fmap _ Miss    = Miss
-  {-# INLINE fmap #-}
 
 -------------------------------------------------------------------------------
 -- Oracles
 -------------------------------------------------------------------------------
 
-newtype Oracle a = Oracle { fromOracle :: forall t. Typeable t => t -> Answer a }
+newtype Oracle a = Oracle { fromOracle :: forall t. Typeable t => t -> Answer t a }
 
-instance Functor Oracle where
-  fmap f (Oracle g) = Oracle (fmap f . g)
-  {-# INLINE fmap #-}
-
-hitTest :: (Data a, Typeable b) => a -> b -> Oracle b
-hitTest a b
-  | kb <- typeOf b = case readCacheFollower (dataBox a) kb of
-    Nothing -> Oracle $ \c ->
-      if typeOf c == kb
-      then Hit (unsafeCoerce c)
-      else Follow
-    Just p -> Oracle $ \c -> let kc = typeOf c in
-      if kc == kb then Hit (unsafeCoerce c)
-      else if p kc then Follow
-      else Miss
+hitTest :: forall a b. (Data a, Typeable b) => a -> b -> Oracle b
+hitTest a b = Oracle $ \(c :: c) ->
+  case mightBe :: Maybe (Is c b) of
+    Just Data.Data.Lens.Refl -> Hit c
+    Nothing ->
+      case readCacheFollower (dataBox a) (typeOf b) of
+        Just p | not (p (typeOf c)) -> Miss
+        _ -> Follow
 
 -------------------------------------------------------------------------------
 -- Traversals
 -------------------------------------------------------------------------------
 
 
-biplateData :: forall f s a. (Applicative f, Data s, Typeable a) => (forall c. Typeable c => c -> Answer a) -> (a -> f a) -> s -> f s
+biplateData :: forall f s a. (Applicative f, Data s, Typeable a) => (forall c. Typeable c => c -> Answer c a) -> (a -> f a) -> s -> f s
 biplateData o f a0 = go2 a0 where
   go :: Data d => d -> f d
   go s = gfoldl (\x y -> x <*> go2 y) pure s
   go2 :: Data d => d -> f d
   go2 s = case o s of
-    Hit a  -> Unsafe.unsafeCoerce <$> f a
+    Hit a  -> f a
     Follow -> go s
     Miss   -> pure s
 {-# INLINE biplateData #-}
 
-uniplateData :: forall f s a. (Applicative f, Data s, Typeable a) => (forall c. Typeable c => c -> Answer a) -> (a -> f a) -> s -> f s
+uniplateData :: forall f s a. (Applicative f, Data s, Typeable a) => (forall c. Typeable c => c -> Answer c a) -> (a -> f a) -> s -> f s
 uniplateData o f a0 = go a0 where
   go :: Data d => d -> f d
   go s = gfoldl (\x y -> x <*> go2 y) pure s
   go2 :: Data d => d -> f d
   go2 s = case o s of
-    Hit a  -> Unsafe.unsafeCoerce <$> f a
+    Hit a  -> f a
     Follow -> go s
     Miss   -> pure s
 {-# INLINE uniplateData #-}
@@ -451,5 +439,3 @@ follower a b m
   | S.size hit < S.size miss = S.member ?? hit
   | otherwise = \k -> not (S.member k miss)
   where (hit, miss) = part (\x -> S.member b (m ! x)) (S.insert a (m ! a))
-
-#endif
