@@ -1,5 +1,7 @@
 {-# LANGUAGE CPP #-}
 {-# LANGUAGE FlexibleContexts #-}
+{-# LANGUAGE FlexibleInstances #-}
+{-# LANGUAGE TypeSynonymInstances #-}
 #ifdef TRUSTWORTHY
 {-# LANGUAGE Trustworthy #-}
 #endif
@@ -14,7 +16,9 @@
 --
 ----------------------------------------------------------------------------
 module Data.Text.Lens
-  ( IsText(..), unpacked
+  ( IsText(..)
+  , unpacked
+  , _Text
   ) where
 
 import           Control.Lens
@@ -51,6 +55,14 @@ class IsText t where
   text = unpacked . traversed
   {-# INLINE text #-}
 
+instance IsText String where
+  packed = id
+  {-# INLINE packed #-}
+  text = indexing traverse
+  {-# INLINE text #-}
+  builder = Lazy.packed . builder
+  {-# INLINE builder #-}
+
 -- | This isomorphism can be used to 'unpack' (or 'pack') both strict or lazy 'Text'.
 --
 -- @
@@ -68,6 +80,18 @@ unpacked :: IsText t => Iso' t String
 unpacked = from packed
 {-# INLINE unpacked #-}
 
+-- | This is an alias for 'unpacked' that makes it clearer how to use it with @('#')@.
+--
+-- @
+-- '_Text' = 'from' 'packed'
+-- @
+--
+-- >>> _Text # "hello" :: Strict.Text
+-- "hello"
+_Text :: IsText t => Iso' t String
+_Text = from packed
+{-# INLINE _Text #-}
+
 instance IsText Strict.Text where
   packed = Strict.packed
   {-# INLINE packed #-}
@@ -83,4 +107,3 @@ instance IsText Lazy.Text where
   {-# INLINE builder #-}
   text = Lazy.text
   {-# INLINE text #-}
-
