@@ -22,6 +22,7 @@ module Numeric.Lens
   , multiplying
   , dividing
   , exponentiating
+  , negated
   ) where
 
 import Control.Lens
@@ -46,6 +47,8 @@ integral = prism toInteger $ \ i -> let a = fromInteger i in
 
 -- | A prism that shows and reads integers in base-2 through base-36
 --
+-- Note: This is an improper prism, since leading 0s are stripped when reading.
+--
 -- >>> "100" ^? base 16
 -- Just 256
 --
@@ -159,8 +162,19 @@ dividing n = iso (/n) (*n)
 --
 -- Note: This errors for n = 0
 --
--- >>> au (exponentiating 2._Unwrapping Sum) (foldMapOf each) (3,4)
--- 5.0
+-- >>> au (_Wrapping Sum . from (exponentiating 2)) (foldMapOf each) (3,4) == 5
+-- True
 exponentiating :: (Floating a, Eq a) => a -> Iso' a a
 exponentiating 0 = error "Numeric.Lens.exponentiating: exponent 0"
 exponentiating n = iso (**n) (**recip n)
+
+
+-- | @'negated' = 'iso' 'negate' 'negate'@
+--
+-- >>> au (_Wrapping Sum . negated) (foldMapOf each) (3,4) == 7
+-- True
+--
+-- >>> au (_Wrapping Sum) (foldMapOf (each.negated)) (3,4) == -7
+-- True
+negated :: Num a => Iso' a a
+negated = iso negate negate
