@@ -1,7 +1,7 @@
 -----------------------------------------------------------------------------
 -- |
 -- Module      :  System.FilePath.Lens
--- Copyright   :  (C) 2012-13 Edward Kmett
+-- Copyright   :  (C) 2012-14 Edward Kmett
 -- License     :  BSD-style (see the file LICENSE)
 -- Maintainer  :  Edward Kmett <ekmett@gmail.com>
 -- Stability   :  experimental
@@ -11,8 +11,8 @@
 module System.FilePath.Lens
   (
   -- * Operators
-    (</>~), (<</>~), (<.>~), (<<.>~)
-  , (</>=), (<</>=), (<.>=), (<<.>=)
+    (</>~), (<</>~), (<<</>~), (<.>~), (<<.>~), (<<<.>~)
+  , (</>=), (<</>=), (<<</>=), (<.>=), (<<.>=), (<<<.>=)
   -- * Lenses
   , basename, directory, extension, filename
   ) where
@@ -31,8 +31,8 @@ import Control.Lens hiding ((<.>))
 -- $setup
 -- >>> :set -XNoOverloadedStrings
 
-infixr 4 </>~, <</>~, <.>~, <<.>~
-infix 4 </>=, <</>=, <.>=, <<.>=
+infixr 4 </>~, <</>~, <<</>~, <.>~, <<.>~, <<<.>~
+infix 4 </>=, <</>=, <<</>=, <.>=, <<.>=, <<<.>=
 
 -- | Modify the path by adding another path.
 --
@@ -83,6 +83,14 @@ l <</>= r = l <%= (</> r)
 {-# INLINE (<</>=) #-}
 
 
+(<<</>~) :: Optical' (->) q ((,)FilePath) s FilePath -> FilePath -> q s (FilePath, s)
+l <<</>~ b = l $ \a -> (a, a </> b)
+{-# INLINE (<<</>~) #-}
+
+(<<</>=) :: MonadState s m => LensLike' ((,)FilePath) s FilePath -> FilePath -> m FilePath
+l <<</>= b = l %%= \a -> (a, a </> b)
+{-# INLINE (<<</>=) #-}
+
 -- | Modify the path by adding extension.
 --
 -- >>> both <.>~ "txt" $ ("hello","world")
@@ -127,7 +135,7 @@ l <<.>~ m = l <%~ (<.> m)
 -- | Add an extension onto the end of the target of a 'Lens' into
 -- your monad's state and return the result.
 --
--- >>> evalState (_1 <<.>= "txt") $("hello","world")
+-- >>> evalState (_1 <<.>= "txt") ("hello","world")
 -- "hello.txt"
 --
 -- When you do not need the result of the operation, ('<.>=') is more flexible.
@@ -136,8 +144,20 @@ l <<.>= r = l <%= (<.> r)
 {-# INLINE (<<.>=) #-}
 
 
+(<<<.>~) :: Optical' (->) q ((,)FilePath) s FilePath -> String -> q s (FilePath, s)
+l <<<.>~ b = l $ \a -> (a, a <.> b)
+{-# INLINE (<<<.>~) #-}
+
+(<<<.>=) :: MonadState s m => LensLike' ((,)FilePath) s FilePath -> String -> m FilePath
+l <<<.>= b = l %%= \a -> (a, a <.> b)
+{-# INLINE (<<<.>=) #-}
+
 -- | A 'Lens' for reading and writing to the basename
 --
+-- Note: This is 'not' a legal 'Lens' unless the outer 'FilePath' has both a directory
+-- and filename component and the generated basenames are not null and contain no directory
+-- separators.
+--
 -- >>> basename .~ "filename" $ "path/name.png"
 -- "path/filename.png"
 basename :: Lens' FilePath FilePath
@@ -147,6 +167,9 @@ basename f p = (<.> takeExtension p) . (takeDirectory p </>) <$> f (takeBaseName
 
 -- | A 'Lens' for reading and writing to the directory
 --
+-- Note: this is /not/ a legal 'Lens' unless the outer 'FilePath' already has a directory component,
+-- and generated directories are not null.
+--
 -- >>> "long/path/name.txt" ^. directory
 -- "long/path"
 directory :: Lens' FilePath FilePath
@@ -156,6 +179,10 @@ directory f p = (</> takeFileName p) <$> f (takeDirectory p)
 
 -- | A 'Lens' for reading and writing to the extension
 --
+-- Note: This is /not/ a legal 'Lens', unless you are careful to ensure that generated
+-- extension 'FilePath' components are either null or start with 'System.FilePath.extSeparator'
+-- and do not contain any internal 'System.FilePath.extSeparator's.
+--
 -- >>> extension .~ ".png" $ "path/name.txt"
 -- "path/name.png"
 extension :: Lens' FilePath FilePath
@@ -167,6 +194,10 @@ extension f p = (n <.>) <$> f e
 
 -- | A 'Lens' for reading and writing to the full filename
 --
+-- Note: This is /not/ a legal 'Lens', unless you are careful to ensure that generated
+-- filename 'FilePath' components are not null and do not contain any
+-- elements of 'System.FilePath.pathSeparators's.
+--
 -- >>> filename .~ "name.txt" $ "path/name.png"
 -- "path/name.txt"
 filename :: Lens' FilePath FilePath
